@@ -9,9 +9,9 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-. lib/inittest
+SKIP_WITH_LVMPOLLD=1
 
-test -e LOCAL_LVMPOLLD && skip
+. lib/inittest
 
 which mkfs.ext3 || skip
 aux have_raid 1 3 0 || skip
@@ -23,6 +23,7 @@ lvcreate -aey --type raid1 -m 3 --ignoremonitoring -L 1 -n 4way $vg
 lvchange --monitor y $vg/4way
 lvs -a -o all,lv_modules $vg
 lvdisplay --maps $vg
+aux wait_for_sync $vg 4way
 aux disable_dev "$dev2" "$dev4"
 mkfs.ext3 "$DM_DEV_DIR/$vg/4way"
 sleep 10 # FIXME: need a "poll" utility, akin to "check"
